angular.module('avatarModule')
  .controller('AvatarControlsController', ['$log', '$window', '$document', 'AvatarOptions', 'AvatarEnums', 'FileSaver',
    'Blob',
    function ($log, $window, $document, AvatarOptions, AvatarEnums, FileSaver, Blob) {
      this.initialize = () => {
        this.formFields = this.generateFormFields();

        // need to initialize avatar values to something
        // otherwise component will fail
        this.randomizeAvatar();
      };

      this.randomizeAvatar = () => {
        AvatarOptions.setRandom();
        this.formModel = AvatarOptions.get();
      };

      this.updateAvatar = () => {
        AvatarOptions.setAll(this.formModel);
      };

      this.generateFormFields = () => {
        const enums = Object.entries(AvatarEnums).map((e) => ({ label: e[0], options: e[1] }));
        enums.forEach(
          (e) => {
            const options = Object.entries(e.options).map((o) => ({ name: o[0], value: o[1] }));
            e.options = options;
          },
        );
        return enums;
      };

      this.shouldShowField = (label) => {
        switch (label) {
          case 'accessories':
            return this.showAccessories();
          case 'backgroundColor':
            return this.showBackgroundColor();
          case 'hairColor':
            return this.showHairColor();
          case 'hatColor':
            return this.showHatColor();
          case 'facialHair':
            return this.showFacialHair();
          case 'facialHairColor':
            return this.showFacialHair() && this.showFacialHairColor();
          case 'clothesColor':
            return this.showClothesColor();
          case 'graphic':
            return this.showGraphics();
          default:
            return true;
        }
      };

      this.showAccessories = () => this.formModel.top !== AvatarEnums.Top.EYEPATCH;
      this.showBackgroundColor = () => this.formModel.background !== AvatarEnums.Background.TRANSPARENT;
      this.showFacialHair = () => this.formModel.top !== AvatarEnums.Top.HIJAB;
      this.showFacialHairColor = () => this.formModel.facialHair !== AvatarEnums.FacialHair.NONE;
      this.showGraphics = () => this.formModel.clothes === AvatarEnums.Clothes.GRAPHIC_SHIRT;
      this.showClothesColor = () => {
        const {
          BLAZER_SHIRT, BLAZER_SWEATER,
        } = AvatarEnums.Clothes;
        const notInclude = [
          BLAZER_SHIRT,
          BLAZER_SWEATER,
        ];
        return !notInclude.includes(this.formModel.clothes);
      };

      this.showHatColor = () => {
        const {
          HIJAB, WINTER_HAT1, WINTER_HAT2, WINTER_HAT3, WINTER_HAT4, TURBAN,
        } = AvatarEnums.Top;
        const include = [
          HIJAB,
          TURBAN,
          WINTER_HAT1,
          WINTER_HAT2,
          WINTER_HAT3,
          WINTER_HAT4,
        ];
        return include.includes(this.formModel.top);
      };

      this.showHairColor = () => {
        const {
          HIJAB, WINTER_HAT1, WINTER_HAT2, WINTER_HAT3, WINTER_HAT4, TURBAN, BALD, HAT, EYEPATCH,
        } = AvatarEnums.Top;
        const notInclude = [
          HIJAB,
          TURBAN,
          WINTER_HAT1,
          WINTER_HAT2,
          WINTER_HAT3,
          WINTER_HAT4,
          BALD,
          HAT,
          EYEPATCH,
        ];
        return !notInclude.includes(this.formModel.top);
      };

      this.downloadSVG = () => {
        const svgNode = $document[0].getElementById('avatar');
        const data = svgNode.innerHTML;
        const svg = new Blob([data], { type: 'image/svg+xml' });
        FileSaver.saveAs(svg, 'avatar.svg');
      };

      this.downloadPNG = () => {
        const svgNode = $document[0].getElementById('avatar');
        const canvas = $document[0].getElementById('canvasRef');
        canvas.width = 400;
        canvas.height = 408;
        const ctx = canvas.getContext('2d');
        ctx.clearRect(0, 0, canvas.width, canvas.height);

        const DOMURL = $window.URL || $window.webkitURL || $window;
        const data = svgNode.innerHTML;
        const svg = new Blob([data], { type: 'image/svg+xml' });
        // eslint-disable-next-line no-undef
        const img = new Image(canvas.width, canvas.height);
        const url = DOMURL.createObjectURL(svg);
        img.onload = () => {
          ctx.save();
          ctx.drawImage(img, 0, 0, canvas.width, canvas.height);
          ctx.restore();
          DOMURL.revokeObjectURL(url);
          canvas.toBlob((imageBlob) => {
            FileSaver.saveAs(imageBlob, 'avatar.png');
          });
        };
        img.src = url;
      };

      this.initialize();
    },
  ]);
